@@ -1,11 +1,10 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemakjan@science.uva.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2005, University of Amsterdam
+    Copyright (C): 1985-2012, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -51,7 +50,8 @@
 user:file_search_path(user_profile, '.').
 user:file_search_path(user_profile, app_preferences('.')).
 :- if(current_prolog_flag(windows, true)).
-user:file_search_path(app_preferences, PrologAppData) :-
+user:file_search_path(app_preferences, app_data('.')).
+user:file_search_path(app_data, PrologAppData) :-
 	current_prolog_flag(windows, true),
 	catch(win_folder(appdata, AppData), _, fail),
 	atom_concat(AppData, '/SWI-Prolog', PrologAppData),
@@ -59,6 +59,9 @@ user:file_search_path(app_preferences, PrologAppData) :-
 	->  true
 	;   catch(make_directory(PrologAppData), _, fail)
 	).
+:- else.
+user:file_search_path(app_data, UserLibDir) :-
+	catch(expand_file_name('~/lib/swipl', [UserLibDir]), _, fail).
 :- endif.
 user:file_search_path(app_preferences, UserHome) :-
 	catch(expand_file_name(~, [UserHome]), _, fail).
@@ -155,7 +158,7 @@ initialization(Goal) :-
 	initialization(Goal, after_load).
 
 :- multifile
-	prolog:initialization_now/2,
+	prolog:initialize_now/2,
 	prolog:message//1.
 
 prolog:initialize_now(load_foreign_library(_),
@@ -333,12 +336,11 @@ load_associated_file :-
 	load_files(user:File, [expand(false)]).
 load_associated_file.
 
-
+:- if(current_predicate(system:win_registry_get_value/3)).
 hkey('HKEY_CURRENT_USER/Software/SWI/Prolog').
 hkey('HKEY_LOCAL_MACHINE/Software/SWI/Prolog').
 
 '$set_prolog_file_extension' :-
-	'$c_current_predicate'(_, system:win_registry_get_value(_,_,_)),
 	hkey(Key),
 	catch(win_registry_get_value(Key, fileExtension, Ext0),
 	      _, fail), !,
@@ -347,6 +349,7 @@ hkey('HKEY_LOCAL_MACHINE/Software/SWI/Prolog').
 	;   Ext = Ext0
 	),
 	create_prolog_flag(associate, Ext, []).
+:- endif.
 '$set_prolog_file_extension'.
 
 
@@ -368,18 +371,14 @@ initialise_prolog :-
 	'$clean_history',
 	set_associated_file,
 	'$set_file_search_paths',
-	once(print_predicate(_, [print], PrintOptions)),
-	create_prolog_flag(toplevel_print_options, PrintOptions, []),
-	create_prolog_flag(prompt_alternatives_on, determinism, []),
-	create_prolog_flag(toplevel_extra_white_line, true, []),
-	create_prolog_flag(toplevel_print_factorized, false, []),
-	'$set_debugger_print_options'(print),
+	init_debug_flags,
 	'$run_initialization',
 	'$load_system_init_file',
+	start_pldoc,
+	attach_packs,
 	'$option'(init_file, OsFile),
 	prolog_to_os_filename(File, OsFile),
 	'$load_init_file'(File),
-	start_pldoc,
 	'$load_script_file',
 	load_associated_file,
 	'$option'(goal, GoalAtom),
@@ -390,6 +389,37 @@ initialise_prolog :-
 	),
 	ignore(user:TheGoal).
 
+init_debug_flags :-
+	once(print_predicate(_, [print], PrintOptions)),
+	create_prolog_flag(toplevel_print_options, PrintOptions, []),
+	create_prolog_flag(prompt_alternatives_on, determinism, []),
+	create_prolog_flag(toplevel_extra_white_line, true, []),
+	create_prolog_flag(toplevel_print_factorized, false, []),
+	'$set_debugger_print_options'(print).
+
+%%	setup_colors is det.
+%
+%	Setup  interactive  usage  by  enabling    colored   output.
+
+setup_colors :-
+	(   stream_property(user_output, tty(true)),
+	    \+ current_prolog_flag(color_term, false)
+	->  catch(load_files(user:library(ansi_term),
+			     [silent(true), if(not_loaded)]),
+		  _, true)
+	;   true
+	).
+
+setup_history :-
+	(   stream_property(user_input, tty(true)),
+	    current_predicate(rl_add_history/1),
+	    \+ current_prolog_flag(save_history, false),
+	    catch(load_files(library(prolog_history), [silent(true)]), _, fail)
+	->  prolog_history(enable)
+	;   true
+	).
+
+
 :- '$hide'('$toplevel'/0).		% avoid in the GUI stacktrace
 
 %%	'$toplevel'/0
@@ -405,8 +435,8 @@ initialise_prolog :-
 %	Actually run the toplevel. If there  is   a  syntax error in the
 %	goal there is no reason to   persue.  Something like that should
 %	happen to repetitive exceptions in the toplevel as well, but how
-%	do we distinguish between a stupid   user and a program crashing
-%	in a loop?
+%	do we distinguish between  interactive   usage  that  frequently
+%	raises and error and a program crashing in a loop?
 %
 %	@see prolog/0 is the default interactive toplevel
 
@@ -418,7 +448,9 @@ initialise_prolog :-
 	toplevel_goal(TopLevel0, TopLevel),
 	user:TopLevel.
 
-toplevel_goal(prolog, '$query_loop') :- !.
+toplevel_goal(prolog, '$query_loop') :- !,
+	setup_colors,
+	setup_history.
 toplevel_goal(Goal, Goal).
 
 
@@ -427,9 +459,9 @@ toplevel_goal(Goal, Goal).
 %	Toplevel called when invoked with -c option.
 
 '$compile' :-
-	'$run_initialization',
-	'$load_system_init_file',
 	'$set_file_search_paths',
+	init_debug_flags,
+	'$run_initialization',
 	catch('$compile_wic', E, (print_message(error, E), halt(1))).
 
 
@@ -437,6 +469,12 @@ toplevel_goal(Goal, Goal).
 		*    USER INTERACTIVE LOOP      *
 		*********************************/
 
+%%	prolog
+%
+%	Run the Prolog toplevel. This is now  the same as break/0, which
+%	pretends  to  be  in  a  break-level    if  there  is  a  parent
+%	environment.
+
 prolog :-
 	break.
 
@@ -461,11 +499,12 @@ prolog :-
 		    prompt(Old, '')
 		),
 		trim_stacks,
-		read_query(Prompt, Goal, Bindings),
+		read_query(Prompt, Query, Bindings),
 		prompt(_, Old),
-		call_expand_query(Goal, ExpandedGoal,
+		call_expand_query(Query, ExpandedQuery,
 				  Bindings, ExpandedBindings)
-	    ->  '$execute'(ExpandedGoal, ExpandedBindings)
+	    ->  expand_goal(ExpandedQuery, Goal),
+	        '$execute'(Goal, ExpandedBindings)
 	    ), !.
 
 
@@ -693,15 +732,16 @@ write_bindings2(Bindings, Residuals, _Det) :-
 %	of VN.
 
 join_same_bindings([], []).
-join_same_bindings([Name=Val|T0], [[Name|Names]=V|T]) :-
-	take_same_bindings(Val, V, T0, Names, T1),
+join_same_bindings([Name=V0|T0], [[Name|Names]=V|T]) :-
+	take_same_bindings(T0, V0, V, Names, T1),
 	join_same_bindings(T1, T).
 
-take_same_bindings(Val, V, [Name=V1|T0], L, T) :-
-	V1 == Val, !,
-	L = [Name|Names],
-	take_same_bindings(V1, V, T0, Names, T).
-take_same_bindings(V, V, L, [], L).
+take_same_bindings([], Val, Val, [], []).
+take_same_bindings([Name=V1|T0], V0, V, [Name|Names], T) :-
+	V0 == V1, !,
+	take_same_bindings(T0, V1, V, Names, T).
+take_same_bindings([Pair|T0], V0, V, Names, [Pair|T]) :-
+	take_same_bindings(T0, V0, V, Names, T).
 
 
 %%	omit_qualifiers(+QGoals, +TypeIn, -Goals) is det.
