@@ -79,17 +79,19 @@ menu('&Debug',
 menu('&Help',
      [ '&About ...'				= about,
        '&Help ...'				= help,
-       'SWI-Prolog &Manual (on www) ...'	= www_open(swipl_man),
+       'Browse &PlDoc ...'			= doc_browser,
        --,
-       'SWI-Prolog &WWW home (on www) ...'	= www_open(swipl),
-       'SWI-Prolog &FAQ (on www) ...'		= www_open(swipl_faq),
-       'SWI-Prolog &Quick Start (on www) ...'	= www_open(swipl_quick),
-       'SWI-Prolog Mailing &List (on www) ...'	= www_open(swipl_mail),
-       'SWI-Prolog &Download (on www) ...'	= www_open(swipl_download),
+       'SWI-Prolog website ...'			= www_open(swipl),
+       '  &Manual ...'				= www_open(swipl_man),
+       '  &FAQ ...'				= www_open(swipl_faq),
+       '  &Quick Start ...'			= www_open(swipl_quick),
+       '  Mailing &List ...'			= www_open(swipl_mail),
+       '  &Download ...'			= www_open(swipl_download),
+       '  &Extension packs ...'			= www_open(swipl_pack),
        --,
        '&XPCE (GUI) Manual ...'			= manpce,
        --,
-       'Submit &Bug report (on www) ...'	= www_open(swipl_bugs)
+       'Submit &Bug report ...'			= www_open(swipl_bugs)
      ],
      [ before_menu(-)
      ]).
