@@ -1,11 +1,9 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker and Anjo Anjewierden
-    E-mail:        J.Wielemaker@cs.vu.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2010, University of Amsterdam
+    Copyright (C): 1985-2013, University of Amsterdam
 			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
@@ -37,6 +35,7 @@
 					% Basic output routines
 	    page//1,			% :Content
 	    page//2,			% :Head, :Body
+	    page//3,			% +Style, :Head, :Body
 	    html//1,			% :Content
 
 					% Option processing
@@ -49,9 +48,11 @@
 	    html_receive//2,		% +Id, :Handler
 	    xhtml_ns//2,		% +Id, +Value
 
+	    html/4,			% <![html[quasi quotations]]>
+
 					% Useful primitives for expanding
 	    html_begin//1,		% +EnvName[(Attribute...)]
-	    html_end//1,			% +EnvName
+	    html_end//1,		% +EnvName
 	    html_quoted//1,		% +Text
 	    html_quoted_attribute//1,	% +Attribute
 
@@ -71,6 +72,8 @@
 :- use_module(library(pairs)).
 :- use_module(library(sgml)).		% Quote output
 :- use_module(library(uri)).
+:- use_module(html_quasiquotations).
+
 :- set_prolog_flag(generate_debug_info, false).
 
 :- meta_predicate
@@ -84,6 +87,11 @@
 	html_receive(+, 3, -, +),
 	html_post(+, :, -, +).
 
+:- multifile
+	expand//1,			% +HTMLElement
+	expand_attribute_value//1.	% +HTMLAttributeValue
+
+
 /** <module> Write HTML text
 
 The purpose of this library  is  to   simplify  writing  HTML  pages. Of
@@ -121,23 +129,26 @@ encoding.
 %%	html_set_options(+Options) is det.
 %
 %	Set options for the HTML output.   Options  are stored in prolog
-%	flags to ensure  with  proper   multi-threaded  behaviour  where
-%	setting an option is local to the   thread and new threads start
-%	with the options from the parent thread.  Defined options are:
+%	flags to ensure proper multi-threaded behaviour where setting an
+%	option is local to the thread  and   new  threads start with the
+%	options from the parent thread. Defined options are:
 %
-%		* dialect(Dialect)
-%		One of =html= (default) or =xhtml=.
+%	  * dialect(Dialect)
+%	    One of =html4=, =xhtml= or =html5= (default). For
+%	    compatibility reasons, =html= is accepted as an
+%	    alias for =html4=.
 %
-%		* doctype(+DocType)
-%		Set the =|<|DOCTYPE|= DocType =|>|= line for page//1 and
-%		page//2.
+%	  * doctype(+DocType)
+%	    Set the =|<|DOCTYPE|= DocType =|>|= line for page//1 and
+%	    page//2.
 %
-%		* content_type(+ContentType)
-%		Set the =|Content-type|= for reply_html_page/3
+%	  * content_type(+ContentType)
+%	    Set the =|Content-type|= for reply_html_page/3
 %
-%	Note  that  the  doctype  is  covered    by  two  prolog  flags:
-%	=html_doctype= for the html dialect  and =xhtml_doctype= for the
-%	xhtml dialect. Dialect muct be switched before doctype.
+%	Note that the doctype and  content_type   flags  are  covered by
+%	distinct  prolog  flags:  =html4_doctype=,  =xhtml_doctype=  and
+%	=html5_doctype= and similar for the   content  type. The Dialect
+%	must be switched before doctype and content type.
 
 html_set_options(Options) :-
 	must_be(list, Options),
@@ -148,24 +159,27 @@ set_options([H|T]) :-
 	html_set_option(H),
 	set_options(T).
 
-html_set_option(dialect(Dialect)) :- !,
-	must_be(oneof([html,xhtml]), Dialect),
+html_set_option(dialect(Dialect0)) :- !,
+	must_be(oneof([html,html4,xhtml,html5]), Dialect0),
+	(   html_version_alias(Dialect0, Dialect)
+	->  true
+	;   Dialect = Dialect0
+	),
 	set_prolog_flag(html_dialect, Dialect).
 html_set_option(doctype(Atom)) :- !,
 	must_be(atom, Atom),
-	(   current_prolog_flag(html_dialect, html)
-	->  set_prolog_flag(html_doctype, Atom)
-	;   set_prolog_flag(xhtml_doctype, Atom)
-	).
+	current_prolog_flag(html_dialect, Dialect),
+	dialect_doctype_flag(Dialect, Flag),
+	set_prolog_flag(Flag, Atom).
 html_set_option(content_type(Atom)) :- !,
 	must_be(atom, Atom),
-	(   current_prolog_flag(html_dialect, html)
-	->  set_prolog_flag(html_content_type, Atom)
-	;   set_prolog_flag(xhtml_content_type, Atom)
-	).
+	current_prolog_flag(html_dialect, Dialect),
+	dialect_content_type_flag(Dialect, Flag),
+	set_prolog_flag(Flag, Atom).
 html_set_option(O) :-
 	domain_error(html_option, O).
 
+html_version_alias(html, html4).
 
 %%	html_current_option(?Option) is nondet.
 %
@@ -174,26 +188,34 @@ html_set_option(O) :-
 html_current_option(dialect(Dialect)) :-
 	current_prolog_flag(html_dialect, Dialect).
 html_current_option(doctype(DocType)) :-
-	(   current_prolog_flag(html_dialect, html)
-	->  current_prolog_flag(html_doctype, DocType)
-	;   current_prolog_flag(xhtml_doctype, DocType)
-	).
+	current_prolog_flag(html_dialect, Dialect),
+	dialect_doctype_flag(Dialect, Flag),
+	current_prolog_flag(Flag, DocType).
 html_current_option(content_type(ContentType)) :-
-	(   current_prolog_flag(html_dialect, html)
-	->  current_prolog_flag(html_content_type, ContentType)
-	;   current_prolog_flag(xhtml_content_type, ContentType)
-	).
+	current_prolog_flag(html_dialect, Dialect),
+	dialect_content_type_flag(Dialect, Flag),
+	current_prolog_flag(Flag, ContentType).
 
+dialect_doctype_flag(html4, html4_doctype).
+dialect_doctype_flag(html5, html5_doctype).
+dialect_doctype_flag(xhtml, xhtml_doctype).
 
-option_default(html_dialect, html).
-option_default(html_doctype,
+dialect_content_type_flag(html4, html4_content_type).
+dialect_content_type_flag(html5, html5_content_type).
+dialect_content_type_flag(xhtml, xhtml_content_type).
+
+option_default(html_dialect, html5).
+option_default(html4_doctype,
 	       'HTML PUBLIC "-//W3C//DTD HTML 4.01 Transitional//EN" \c
 	       "http://www.w3.org/TR/html4/loose.dtd"').
+option_default(html5_doctype,
+	       'html').
 option_default(xhtml_doctype,
 	       'html PUBLIC "-//W3C//DTD XHTML 1.0 \c
 	       Transitional//EN" \c
 	       "http://www.w3.org/TR/xhtml1/DTD/xhtml1-transitional.dtd"').
-option_default(html_content_type, 'text/html; charset=UTF-8').
+option_default(html4_content_type, 'text/html; charset=UTF-8').
+option_default(html5_content_type, 'text/html; charset=UTF-8').
 option_default(xhtml_content_type, 'application/xhtml+xml; charset=UTF-8').
 
 %%	init_options is det.
@@ -272,6 +294,9 @@ content_type -->
 	html_post(head, meta([ 'http-equiv'('content-type'),
 			       content(Type)
 			     ], [])).
+content_type -->
+	{ html_current_option(dialect(html5)) }, !,
+	html_post(head, meta('charset=UTF-8')).
 content_type -->
 	[].
 
@@ -323,27 +348,24 @@ hook_module(_, user, PI) :-
 
 html(Spec) -->
 	{ strip_module(Spec, M, T) },
-	html(T, M).
+	qhtml(T, M).
 
-html([], _) --> !,
+qhtml([], _) --> !,
 	[].
-html([H|T], M) --> !,
+qhtml([H|T], M) --> !,
 	html_expand(H, M),
-	html(T, M).
-html(X, M) -->
+	qhtml(T, M).
+qhtml(X, M) -->
 	html_expand(X, M).
 
 html_expand(M:Term, _) --> !,
-	html(Term, M).
+	qhtml(Term, M).
 html_expand(Term, Module) -->
 	do_expand(Term, Module), !.
 html_expand(Term, _Module) -->
 	{ print_message(error, html(expand_failed(Term))) }.
 
 
-:- multifile
-	expand/3.
-
 do_expand(Token, _) -->			% call user hooks
 	expand(Token), !.
 do_expand(Fmt-Args, _) --> !,
@@ -357,7 +379,7 @@ do_expand(\List, Module) -->
 do_expand(\Term, Module, In, Rest) :- !,
 	call(Module:Term, In, Rest).
 do_expand(Module:Term, _) --> !,
-	html(Term, Module).
+	qhtml(Term, Module).
 do_expand(script(Content), _) --> !,	% general CDATA declared content elements?
 	html_begin(script),
 	[ Content
@@ -379,7 +401,7 @@ do_expand(element(Env, Attributes, Contents), M) --> !,
 	    }
 	->  xhtml_empty(Env, Attributes)
 	;   html_begin(Env, Attributes),
-	    html(Contents, M),
+	    qhtml(Contents, M),
 	    html_end(Env)
 	).
 do_expand(Term, M) -->
@@ -393,7 +415,7 @@ do_expand(Term, M) -->
 		}
 	    ->  xhtml_empty(Env, [])
 	    ;	html_begin(Env),
-		html(Contents, M),
+		qhtml(Contents, M),
 		html_end(Env)
 	    )
 	).
@@ -406,14 +428,15 @@ do_expand(Term, M) -->
 	    }
 	->  xhtml_empty(Env, Attributes)
 	;   html_begin(Env, Attributes),
-	    html(Contents, M),
+	    qhtml(Contents, M),
 	    html_end(Env)
 	).
 
 check_non_empty([], _, _) :- !.
 check_non_empty(_, Tag, Term) :-
 	layout(Tag, _, empty), !,
-	print_message(warning, format('Using empty element with content: ~p', [Term])).
+	print_message(warning,
+		      format('Using empty element with content: ~p', [Term])).
 check_non_empty(_, _, _).
 
 %%	raw(+List, +Modules)// is det.
@@ -960,6 +983,8 @@ layout(h2,	   2-0,	0-2).
 layout(h3,	   2-0,	0-2).
 layout(h4,	   2-0,	0-2).
 
+layout(iframe,	   1-1, 1-1).
+
 layout(hr,	   1-1, empty).		% empty elements
 layout(br,	   0-1, empty).
 layout(img,	   0-0, empty).
@@ -1208,7 +1233,7 @@ prolog_colour:goal_colours(Goal, Colours) :-
 					% TBD: Check with do_expand!
 html_colours(Var, classify) :-
 	var(Var), !.
-html_colours(\List, built_in-Colours) :-
+html_colours(\List, built_in-[built_in-Colours]) :-
 	is_list(List), !,
 	list_colours(List, Colours).
 html_colours(\_, built_in-[dcg]) :- !.
