@@ -1,11 +1,10 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        wielemak@science.uva.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2006, University of Amsterdam
+    Copyright (C): 1985-2013, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -19,7 +18,7 @@
 
     You should have received a copy of the GNU General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -40,22 +39,39 @@
 :- use_module(library(sgml)).
 :- use_module(library(occurs)).
 :- use_module(library(lists)).
-:- use_module(library(url)).
+:- use_module(library(uri)).
 :- use_module(library(apply)).
+:- use_module(library(option)).
+:- use_module(library(filesex)).
 :- use_module(doc_wiki).
 :- use_module(doc_html).
 :- use_module(doc_search).
+:- use_module(doc_process).
 :- use_module(library(http/html_write)).
 :- use_module(library(http/html_head)).
 :- use_module(library(http/http_dispatch)).
 :- use_module(library(http/http_path)).
-:- use_module(library(doc_http)).
 :- include(hooks).
 
 /** <module> Process SWI-Prolog HTML manuals
 
 */
 
+:- predicate_options(index_man_directory/2, 2,
+		     [ class(oneof([manual,packages,misc])),
+		       pass_to(system:absolute_file_name/3, 3)
+		     ]).
+:- predicate_options(man_page//2, 2,
+		     [ for(atom),
+		       links(boolean),
+		       footer(boolean),
+		       no_manual(oneof([fail,error])),
+		       search_in(oneof([all, app, man])),
+		       search_match(oneof([name, summary])),
+		       search_options(boolean)
+		     ]).
+
+
 :- dynamic
 	man_index/5.		% Object, Summary, File, Class, Offset
 
@@ -89,6 +105,10 @@ manual_directory(packages, swi('doc/packages')).
 index_manual :-
 	man_index(_,_,_,_,_), !.
 index_manual :-
+	with_mutex(pldoc_man,
+		   locked_index_manual).
+
+locked_index_manual :-
 	(   manual_directory(Class, Dir),
 	    index_man_directory(Dir,
 				[ class(Class),
@@ -97,6 +117,26 @@ index_manual :-
 	    fail ; true
 	).
 
+check_dubplicate_ids :-
+	findall(Id, man_index(section(_,_,Id,_),_,_,_,_), Ids),
+	msort(Ids, Sorted),
+	duplicate_ids(Sorted, Duplicates),
+	(   Duplicates == []
+	->  true
+	;   print_message(warning, pldoc(duplicate_ids(Duplicates)))
+	).
+
+duplicate_ids([], []).
+duplicate_ids([H,H|T0], [H|D]) :- !,
+	take_prefix(H,T0,T),
+	duplicate_ids(T, D).
+duplicate_ids([_|T], D) :-
+	duplicate_ids(T, D).
+
+take_prefix(H, [H|T0], T) :- !,
+	take_prefix(H, T0, T).
+take_prefix(_, L, L).
+
 
 %%	index_man_directory(Dir, +Options) is det
 %
@@ -108,11 +148,7 @@ index_manual :-
 %	Remaining Options are passed to absolute_file_name/3.
 
 index_man_directory(Spec, Options) :-
-	(   select(class(Class), Options, Options1)
-	->  true
-	;   Options1 = Options,
-	    Class = misc
-	),
+	select_option(class(Class), Options, Options1, misc),
 	absolute_file_name(Spec, Dir,
 			   [ file_type(directory),
 			     access(read)
@@ -188,8 +224,10 @@ index_on_begin(div, Attributes, Parser) :- !,
 		   ]),
 	dom_to_text(DOM, Title),
 	nb_getval(pldoc_index_class, Class),
-	assert(man_index(section(0, '0', File), Title, File, Class, Offset)).
-index_on_begin(H, _, Parser) :-		% TBD: add class for document title.
+	swi_local_path(File, Local),
+	assert(man_index(section(0, '0', Local, File),
+			 Title, File, Class, Offset)).
+index_on_begin(H, Attributes, Parser) :- % TBD: add class for document title.
 	heading(H, Level),
 	get_sgml_parser(Parser, charpos(Offset)),
         get_sgml_parser(Parser, file(File)),
@@ -199,7 +237,17 @@ index_on_begin(H, _, Parser) :-		% TBD: add class for document title.
 		   ]),
 	dom_section(Doc, Nr, Title),
 	nb_getval(pldoc_index_class, Class),
-	assert(man_index(section(Level, Nr, File), Title, File, Class, Offset)).
+	section_id(Attributes, Title, File, ID),
+	assert(man_index(section(Level, Nr, ID, File),
+			 Title, File, Class, Offset)).
+
+section_id(Attributes, _Title, _, ID) :-
+	memberchk(id=ID, Attributes), !.
+section_id(_Attributes, Title, File, ID) :-
+	atomic_list_concat(Words, ' ', Title),
+	atomic_list_concat(Words, '_', ID0),
+	atom_concat('sec:', ID0, ID),
+	print_message(warning, pldoc(no_section_id(File, Title))).
 
 %%	dom_section(+HeaderDOM, -NR, -Title) is semidet.
 %
@@ -227,7 +275,7 @@ section_number(Title, Nr, PlainTitle) :-
 	(   char_type(Start, digit)
 	->  true
 	;   char_type(Start, upper),
-	    sub_atom(Title, 1, 1, _, '.') 	% A., etc: Appendices
+	    sub_atom(Title, 1, 1, _, '.')	% A., etc: Appendices
 	),
 	sub_atom(Title, B, _, A, ' '), !,
 	sub_atom(Title, 0, B, _, Nr),
@@ -327,7 +375,7 @@ cdata(_) -->
 
 load_man_object(Obj, ParentSection, Path, DOM) :-
 	resolve_section(Obj, For),
-	For = section(_,SN,Path),
+	For = section(_,SN,_ID,Path),
 	parent_section(For, ParentSection),
 	findall(Nr-Pos, section_start(Path, Nr, Pos), Pairs),
 	(   (   Pairs = [SN-_|_]
@@ -400,7 +448,7 @@ load_man_object(For, Parent, Path, DOM) :-
 
 section_start(Path, Nr, Pos) :-
 	index_manual,
-	man_index(section(_,Nr,_), _, Path, _, Pos).
+	man_index(section(_,Nr,_,_), _, Path, _, Pos).
 
 %%	resolve_section(+SecIn, -SecOut) is det.
 %
@@ -408,18 +456,28 @@ section_start(Path, Nr, Pos) :-
 %	number if this information is missing.   The latter allows us to
 %	refer to files of the manual.
 
-resolve_section(section(Level, No, Spec),
-		section(Level, No, Path)) :-
+resolve_section(section(Level, No, Spec), Section) :- !,
+	resolve_section(section(Level, No, _, Spec), Section).
+resolve_section(section(Level, No, ID, Path),
+		section(Level, No, ID, Path)) :-
+	nonvar(ID),
+	index_manual,
+	man_index(section(Level,No,ID,Path), _, _, _, _), !.
+resolve_section(section(Level, No, ID, Spec),
+		section(Level, No, ID, Path)) :-
+	ground(Spec),
 	absolute_file_name(Spec, Path,
 			   [ access(read)
 			   ]),
-	(   man_index(section(Level, No, Path), _, _, _, _)
+	(   index_manual,
+	    man_index(section(Level, No, ID, Path), _, _, _, _)
 	->  true
 	;   path_allowed(Path)
 	->  true
 	;   permission_error(read, manual_file, Spec)
 	).
 
+
 path_allowed(Path) :-			% allow all files from swi/doc
 	absolute_file_name(swi(doc), Parent,
 			   [ access(read),
@@ -428,15 +486,15 @@ path_allowed(Path) :-			% allow all files from swi/doc
 	sub_atom(Path, 0, _, _, Parent).
 
 
-%%	parent_section(+Section, +Parent) is det.
+%%	parent_section(+Section, -Parent) is det.
 %
 %	Parent is the parent-section  of   Section.  First  computes the
 %	section number and than finds the   required  number in the same
 %	file or same directory.
 
-parent_section(section(Level, Nr, File), Parent) :-
+parent_section(section(Level, Nr, _ID, File), Parent) :-
 	integer(Level),
-	Parent = section(PL, PNr, _PFile),
+	Parent = section(PL, PNr, _PID, _PFile),
 	PL is Level - 1,
 	findall(B, sub_atom(Nr, B, _, _, '.'), BL),
 	last(BL, Before),
@@ -448,7 +506,7 @@ parent_section(section(Level, Nr, File), Parent) :-
 	->  true
 	;   man_index(Parent, _, _, _, _)
 	), !.
-parent_section(section(_, _, File), File).
+parent_section(section(_, _, _, File), File).
 
 %%	object_section(+Path, +Position, -Section) is semidet.
 %
@@ -456,7 +514,7 @@ parent_section(section(_, _, File), File).
 %	last section object before position.
 
 object_section(Path, Pos, Section) :-
-	Section	= section(_,_,_),
+	Section	= section(_,_,_,_),
 	findall(Section,
 	       (man_index(Section, _, Path, _, SecPos), SecPos =< Pos),
 		List),
@@ -480,7 +538,7 @@ object_spec(Atom, PI) :-
 
 
 		 /*******************************
-		 *	      EMIT    		*
+		 *	      EMIT		*
 		 *******************************/
 
 %%	man_page(+Obj, +Options)// is semidet.
@@ -491,10 +549,19 @@ object_spec(Atom, PI) :-
 %
 %	    * Name/Arity
 %	    Predicate indicator: display documentation of the predicate
-
-%	    * section(Level, Number, File)
+%
+%	    * f(Name/Arity)
+%	    display documentation of an arithmetic function
+%
+%	    * c(Function)
+%	    display documentation of a C API function
+%
+%	    * section(Level, Number, Id, File)
 %	    Display a section of the manual
 %
+%	    * sec(DocFile#Id)
+%	    Display a section of the manual (from short form)
+%
 %	Options:
 %
 %		* no_manual(Action)
@@ -505,47 +572,155 @@ object_spec(Atom, PI) :-
 %		If =true= (default), include links to the parent object;
 %		if =false=, just emit the manual material.
 
-man_page(Obj, Options) -->
-	{ findall((Parent+Path)-DOM,
+man_page(Obj0, Options) -->			% sections
+	{ full_page(Obj0, Obj),
+          findall((Parent+Path)-(Obj+DOM),
 		  load_man_object(Obj, Parent, Path, DOM),
 		  Matches),
+	  Matches = [_|_], !,
+	  pairs_keys(Matches, ParentPaths),
 	  Matches = [Parent+Path-_|_]
 	},
 	html_requires(pldoc),
-	(   { option(links(true), Options, true) }
-	->  man_links(Parent, Options),
-	    html(p([]))
-	;   []
+	man_links(ParentPaths, Options),
+	man_matches(Matches, Obj, Options).
+man_page(Obj, Options) -->			% predicates, etc.
+	{ full_object(Obj, Full),
+	  findall(Full-File,
+		  ( doc_comment(Full, File:_, _, _),
+		    \+ private(Full, Options)
+		  ),
+		  Pairs),
+	  Pairs \== [],
+	  pairs_keys(Pairs, Objs)
+	}, !,
+	html_requires(pldoc),
+	(   { Pairs = [_-File] }
+	->  object_page_header(File, Options)
+	;   object_page_header(-, Options)
 	),
-	man_matches(Matches).
-man_page(Obj, Options) -->
-	{ \+ option(no_manual(fail), Options),
-	  term_to_atom(Obj, Atom)
+	objects(Objs, [synopsis(true)|Options]).
+man_page(Obj, Options) -->			% failure
+	{ \+ option(no_manual(fail), Options)
 	},
 	html_requires(pldoc),
-	(   { option(links(true), Options, true) }
-	->  man_links([], [])
-	;   html(p(['No manual entry for ', Atom]))
+	man_links([], Options),
+	html(p(class(noman),
+	       [ 'Sorry, No manual entry for ',
+		 b('~w'-[Obj])
+	       ])).
+
+full_page(Obj, _) :-
+	var(Obj), !, fail.
+full_page(Obj, Obj) :-
+	Obj = section(_,_,_,_), !.
+full_page(section(ID), section(_,_,ID,_)) :- !.
+full_page(Obj, Obj) :-
+	ground(Obj).
+
+
+full_object(M:N/A, M:N/A) :- !.
+full_object(M:N//A, M:N/A2) :-
+	integer(A2), !,
+	A2 is A+2.
+full_object(Name/Arity, _:Name/Arity) :- !.
+full_object(Name//Arity, _:Name/Arity2) :-
+	integer(Arity),
+	Arity2 is Arity+1.
+
+
+%%	man_synopsis(+Text, Parent)
+%
+%	Give synopsis details for a fully specified predicate indicator.
+%	The tricky part is that there   are cases where multiple modules
+%	export the same predicate. We must find   from  the title of the
+%	manual section which library is documented.
+
+:- public
+	man_synopsis//2.		% called from man_match//2
+
+man_synopsis(Text, Parent) -->
+	{ atom_pi(Text, PI) },
+	man_synopsis_2(PI, Parent).
+man_synopsis(Object, Parent) -->
+	man_synopsis_2(Object, Parent).
+
+man_synopsis_2(Name/Arity, Parent) -->
+	{ object_module(Parent, Module),
+	  object_href(Parent, HREF)
+	},
+	object_synopsis(Module:Name/Arity, [href(HREF)]).
+man_synopsis_2(Object, _) -->
+	object_synopsis(Object, []).
+
+%%	object_module(+Section, -Module) is semidet.
+%
+%	Find the module documented by Section.
+%
+%	@tbd This requires that the documented file is loaded. If
+%	not, should we use the title of the section?
+
+object_module(Section, Module) :-
+	man_index(Section, Title, _File, _Class, _Offset),
+	(   once(sub_atom(Title, B, _, _, :)),
+	    sub_atom(Title, 0, B, _, Atom),
+	    catch(term_to_atom(Term, Atom), _, fail),
+	    Term = library(_)
+	->  absolute_file_name(Term, PlFile,
+			       [ file_type(prolog),
+				 access(read),
+				 file_errors(fail)
+			       ]),
+	    module_property(Module, file(PlFile))
 	).
 
-man_matches([]) -->
-	[].
-man_matches([(_Parent+Path)-H|T]) -->
-	dom_list(H, Path),
-	man_matches(T).
+atom_pi(Text, Name/Arity) :-
+	atom(Text),
+	sub_atom(Text, Pre, _, Post, /),
+	sub_atom(Text, _, Post, 0, AA),
+	atom_number(AA, Arity),
+	integer(Arity),
+	Arity >= 0, !,
+	sub_atom(Text, 0, Pre, _, Name).
 
-dom_list([], _) -->
+man_matches([Match], Object, Options) -->
+	{ option(footer(true), Options, true) }, !,
+	man_match(Match, Object),
+	object_page_footer(Object, []).
+man_matches(Matches, Object, _) -->
+	man_matches_list(Matches, Object).
+
+man_matches_list([], _) --> [].
+man_matches_list([H|T], Obj) --> man_match(H, Obj), man_matches_list(T, Obj).
+
+%%	man_match(+Term, +Object)//
+%
+%	If  possible,  insert  the  synopsis  into   the  title  of  the
+%	description.
+
+man_match((Parent+Path)-(Obj+[element(dt,A,C),DD]), Obj) -->
+	dom_list([ element(dt,[],[\man_synopsis(Obj, Parent)]),
+		   element(dt,A,C),
+		   DD
+		 ], Path).
+man_match((_Parent+Path)-(Obj+DOM), Obj) -->
+	dom_list(DOM, Path).
+
+:- html_meta
+	dom_list(html, +, ?, ?).
+
+dom_list(_:[], _) --> !,
 	[].
-dom_list([H|T], Path) -->
+dom_list(M:[H|T], Path) -->
 	dom(H, Path),
-	dom_list(T, Path).
+	dom_list(M:T, Path).
 
 dom(element(E, Atts, Content), Path) --> !,
 	dom_element(E, Atts, Content, Path).
 dom(CDATA, _) -->
 	html(CDATA).
 
-dom_element(a, _, [], _) -->			% Useless back-references
+dom_element(a, _, [], _) --> !,			% Useless back-references
 	[].
 dom_element(a, Att, Content, Path) -->
 	{ memberchk(href=HREF, Att),
@@ -556,6 +731,28 @@ dom_element(a, Att, Content, Path) -->
 	  rewrite_ref(Class, HREF, Path, Myref)
 	}, !,
 	html(a(href(Myref), \dom_list(Content, Path))).
+dom_element(span, Att, [CDATA], _) -->
+	{ memberchk(class='pred-ext', Att),
+	  atom_pi(CDATA, PI),
+	  documented(PI),
+	  http_link_to_id(pldoc_man, [predicate=CDATA], HREF)
+	}, !,
+	html(a(href(HREF), CDATA)).
+dom_element(img, Att0, [], Path) -->
+	{ selectchk(src=Src, Att0, Att1),
+	  current_prolog_flag(home, SWI),
+	  sub_atom(Path, 0, Len, _, SWI),
+	  (   sub_atom(Path, Len, _, _, '/doc/Manual/')
+	  ->  Handler = manual_file
+	  ;   sub_atom(Path, Len, _, _, '/doc/packages/')
+	  ->  Handler = pldoc_package
+	  ), !,
+	  http_link_to_id(Handler, [], ManRef),
+	  atomic_list_concat([ManRef, /, Src], NewPath),
+	  Begin =.. [img, src(NewPath) | Att1]
+	}, !,
+	html_begin(Begin),
+	html_end(img).
 dom_element(div, Att, _, _) -->
 	{ memberchk(class=navigate, Att) }, !.
 dom_element(html, _, Content, Path) --> !,	% do not emit a html for the second time
@@ -572,24 +769,35 @@ dom_element(Name, Attrs, Content, Path) -->
 	dom_list(Content, Path),
 	html_end(Name).
 
+%%	documented(+PI) is semidet.
+%
+%	True if we have documentation about PI
+
+documented(PI) :-
+	index_manual,
+	man_index(PI, _, _, _, _), !.
+documented(PI) :-
+	full_object(PI, Obj),
+	doc_comment(Obj, _, _, _), !.
+
 
 %%	rewrite_ref(+Class, +Ref0, +Path, -ManRef) is semidet.
 %
 %	Rewrite Ref0 from the HTML reference manual format to the server
 %	format. Reformatted:
 %
-%		$ File#Name/Arity :
-%		Local reference using the manual presentation
-%		=|/man?predicate=PI|=.
+%	    $ File#Name/Arity :
+%	    Local reference using the manual presentation
+%	    =|/man?predicate=PI|=.
 %
-%		$ File#sec:NR :
-%		Rewrite to =|section(Level, NT, FilePath)|=
+%	    $ File#sec:NR :
+%	    Rewrite to =|section(Level, NT, ID, FilePath)|=
 %
-%		$ File#flag:Name :
-%		Rewrite to =|section(Level, NT, FilePath)#flag:Name|=
+%	    $ File#flag:Name :
+%	    Rewrite to =|section(Level, NT, ID, FilePath)#flag:Name|=
 %
-%		$ File#Name()
-%		Rewrite to /man/CAPI=Name
+%	    $ File#Name()
+%	    Rewrite to /man/CAPI=Name
 %
 %	@param Class	Class of the <A>.  Supported classes are
 %
@@ -606,16 +814,26 @@ rewrite_ref(pred, Ref0, _, Ref) :-		% Predicate reference
 	sub_atom(Ref0, _, A, 0, Fragment),
 	name_to_object(Fragment, PI),
 	man_index(PI, _, _, _, _),
-	www_form_encode(Fragment, Enc),
+	uri_encoded(query_value, Fragment, Enc),
 	http_location_by_id(pldoc_man, ManHandler),
 	format(string(Ref), '~w?predicate=~w', [ManHandler, Enc]).
+rewrite_ref(function, Ref0, _, Ref) :-		% Arithmetic function reference
+	sub_atom(Ref0, _, _, A, '#'), !,
+	sub_atom(Ref0, _, A, 0, Fragment),
+	name_to_object(Fragment, PI),
+	man_index(PI, _, _, _, _),
+	PI=f(Name/Arity),
+	format(atom(PIName), '~w/~w', [Name,Arity]),
+	uri_encoded(query_value, PIName, Enc),
+	http_location_by_id(pldoc_man, ManHandler),
+	format(string(Ref), '~w?function=~w', [ManHandler, Enc]).
 rewrite_ref(func, Ref0, _, Ref) :-		% C-API reference
 	sub_atom(Ref0, _, _, A, '#'), !,
 	sub_atom(Ref0, _, A, 0, Fragment),
 	name_to_object(Fragment, Obj),
 	man_index(Obj, _, _, _, _),
 	Obj = c(Function),
-	www_form_encode(Function, Enc),
+	uri_encoded(query_value, Function, Enc),
 	http_location_by_id(pldoc_man, ManHandler),
 	format(string(Ref), '~w?CAPI=~w', [ManHandler, Enc]).
 rewrite_ref(sec, Ref0, Path, Ref) :-		% Section inside a file
@@ -627,7 +845,7 @@ rewrite_ref(sec, Ref0, Path, Ref) :-		% Section inside a file
 rewrite_ref(sec, File, Path, Ref) :-		% Section is a file
 	file_directory_name(Path, Dir),
 	atomic_list_concat([Dir, /, File], SecPath),
-	Obj = section(_, _, SecPath),
+	Obj = section(_, _, _, SecPath),
 	man_index(Obj, _, _, _, _), !,
 	object_href(Obj, Ref).
 rewrite_ref(flag, Ref0, Path, Ref) :-
@@ -636,7 +854,7 @@ rewrite_ref(flag, Ref0, Path, Ref) :-
 	sub_atom(Ref0, _, A, 0, Fragment),
 	file_directory_name(Path, Dir),
 	atomic_list_concat([Dir, /, File], SecPath),
-	Obj = section(_, _, SecPath),
+	Obj = section(_, _, _, SecPath),
 	man_index(Obj, _, _, _, _), !,
 	object_href(Obj, Ref1),
 	format(string(Ref), '~w#~w', [Ref1, Fragment]).
@@ -645,12 +863,12 @@ rewrite_ref(flag, Ref0, Path, Ref) :-
 %
 %	If Atom is `Name/Arity', decompose to Name and Arity. No errors.
 
-name_to_object(Atom, Name/Arity) :-
-	atom(Atom),
-	atomic_list_concat([Name, AA], /, Atom),
-	catch(atom_number(AA, Arity), _, fail),
-	integer(Arity),
-	Arity >= 0.
+name_to_object(Atom, Object) :-
+	atom_pi(Atom, Name/Arity),
+	(   atom_concat('f-', FuncName, Name)
+	->  Object = f(FuncName/Arity)
+	;   Object = Name/Arity
+	).
 name_to_object(Atom, c(Function)) :-
 	atom(Atom),
 	sub_atom(Atom, 0, _, _, 'PL_'),
@@ -660,39 +878,44 @@ name_to_object(Atom, c(Function)) :-
 
 %%	referenced_section(+Fragment, +File, +Path, -Section)
 
-referenced_section(Fragment, File, Path, section(Level, Nr, SecPath)) :-
+referenced_section(Fragment, File, Path, section(Level, Nr, ID, SecPath)) :-
 	atom_concat('sec:', Nr, Fragment),
 	(   File == ''
 	->  SecPath = Path
 	;   file_directory_name(Path, Dir),
 	    atomic_list_concat([Dir, /, File], SecPath)
 	),
-	man_index(section(Level, Nr, SecPath), _, _, _, _).
+	man_index(section(Level, Nr, ID, SecPath), _, _, _, _).
 
 
-%%	man_links(+Parent, +Options)// is det.
+%%	man_links(+ParentPaths, +Options)// is det.
 %
 %	Create top link structure for manual pages.
 
-man_links(Parent, Options) -->
-	html(div(class(navhdr),
-		 [ div(class(jump), \man_parent(Parent)),
-		   div(class(search), \search_form(Options)),
-		   br(clear(right))
-		 ])).
+man_links(ParentPaths, Options) -->
+	{ option(links(true), Options, true) }, !,
+	html([ div(class(navhdr),
+		   [ div(class(jump), \man_parent(ParentPaths)),
+		     div(class(search), \search_form(Options)),
+		     br(clear(right))
+		   ]),
+	       p([])
+	     ]).
+man_links(_, _) -->
+	[].
 
-man_parent(Section) -->
-	{ Section = section(_,_,_)
+man_parent(ParentPaths) -->
+	{ maplist(parent_to_section, ParentPaths, [Section|MoreSections]),
+	  maplist(=(Section), MoreSections)
 	}, !,
 	object_ref(Section, [secref_style(number_title)]).
-man_parent(File) -->
-	{ atom(File),
-	  Obj = section(_,_,_),
-	  man_index(Obj, _Title, File, _Class, _Offset)
-	}, !,
-	object_ref(Obj, [secref_style(number_title)]).
-man_parent(_) -->
-	[].
+man_parent(_) --> [].
+
+parent_to_section(X+_, X) :-
+	X = section(_,_,_,_), !.
+parent_to_section(File+_, Section) :-
+	atom(File),
+	man_index(Section, _Title, File, _Class, _Offset), !.
 
 %%	section_link(+Obj, +Options)// is det.
 %
@@ -706,7 +929,7 @@ section_link(Section, Options) -->
 	},
 	section_link(Style, Section, Options).
 
-section_link(number, section(_, Number, _), _Options) --> !,
+section_link(number, section(_, Number, _, _), _Options) --> !,
 	(   {Number == '0'}		% Title.  Package?
 	->  []
 	;   html(['Sec. ', Number])
@@ -716,7 +939,7 @@ section_link(title, Obj, _Options) --> !,
 	},
 	html(Title).
 section_link(_, Obj, _Options) --> !,
-	{ Obj = section(_, Number, _),
+	{ Obj = section(_, Number, _, _),
 	  man_index(Obj, Title, _File, _Class, _Offset)
 	},
 	(   { Number == '0' }
@@ -774,7 +997,7 @@ package_class(true,  pkg_link, _).
 package_class(false, no_pkg_link, _).
 
 current_package(pkg(Title, HREF, HavePackage)) :-
-	man_index(section(0, _, _), Title, File, packages, _),
+	man_index(section(0, _, _, _), Title, File, packages, _),
 	file_base_name(File, FileNoDir),
 	file_name_extension(Base, _, FileNoDir),
 	(   exists_source(library(Base))
@@ -790,16 +1013,22 @@ current_package(pkg(Title, HREF, HavePackage)) :-
 %%	pldoc_package(+Request)
 %
 %	HTTP handler for PlDoc package documentation.  Accepts
-%	/pkg/<package>.html.
+%	/pldoc/package/<package>.{html,gif}.
 
+pldoc_package(Request) :-
+	memberchk(path_info(Img), Request),
+	file_name_extension(_, gif, Img), !,
+	atom_concat('doc/packages/', Img, Path),
+	http_reply_file(swi(Path), [], Request).
 pldoc_package(Request) :-
 	(   memberchk(path_info(PkgDoc), Request),
 	    \+ sub_atom(PkgDoc, _, _, _, /),
-	    Obj = section(0,_,_),
+	    Obj = section(0,_,_,_),
 	    index_manual,
 	    man_index(Obj, Title, File, packages, _),
 	    file_base_name(File, PkgDoc)
-	->  reply_html_page(title(Title),
+	->  reply_html_page(pldoc(package),
+			    title(Title),
 			    \object_page(Obj, []))
 	;   memberchk(path(Path), Request),
 	    existence_error(http_location, Path)
@@ -813,16 +1042,18 @@ pldoc_package(Request) :-
 pldoc_refman(Request) :-
 	memberchk(path_info(Section), Request),
 	\+ sub_atom(Section, _, _, _, /),
-	Obj = section(0,_,_),
+	Obj = section(0,_,_,_),
 	index_manual,
 	man_index(Obj, Title, File, manual, _),
 	file_base_name(File, Section), !,
-	reply_html_page(title(Title),
+	reply_html_page(pldoc(man),
+			title(Title),
 			\object_page(Obj, [])).
 pldoc_refman(Request) :-		% server Contents.html
 	\+ memberchk(path_info(_), Request), !,
-	Section = section(_, _, swi('doc/Manual/Contents.html')),
-	reply_html_page(title('SWI-Prolog manual'),
+	Section = section(_, _, _, swi('doc/Manual/Contents.html')),
+	reply_html_page(pldoc(refman),
+			title('SWI-Prolog manual'),
 			\man_page(Section, [])).
 pldoc_refman(Request) :-
 	memberchk(path(Path), Request),
@@ -838,10 +1069,16 @@ prolog:doc_object_summary(Obj, Class, File, Summary) :-
 	man_index(Obj, Summary, File, Class, _Offset).
 
 prolog:doc_object_page(Obj, Options) -->
-	man_page(Obj, [no_manual(fail)|Options]).
+	man_page(Obj, [no_manual(fail),footer(false)|Options]).
 
 prolog:doc_object_link(Obj, Options) -->
-	{ Obj = section(_,_,_) }, !,
+	{ Obj = section(_,_,_,_) }, !,
+	section_link(Obj, Options).
+prolog:doc_object_link(Obj0, Options) -->
+	{ Obj0 = section(ID),
+	  Obj = section(_Level, _No, ID, _Path),
+	  man_index(Obj, _, _, _, _)
+	}, !,
 	section_link(Obj, Options).
 prolog:doc_object_link(Obj, Options) -->
 	{ Obj = c(Function) }, !,
@@ -851,7 +1088,7 @@ prolog:doc_category(manual,   30, 'SWI-Prolog Reference Manual').
 prolog:doc_category(packages, 40, 'Package documentation').
 
 prolog:doc_file_index_header(File, Options) -->
-	{ Section = section(_Level, _No, File),
+	{ Section = section(_Level, _No, _ID, File),
 	  man_index(Section, _Summary, File, _Cat, _Offset)
 	}, !,
 	html(tr(th([colspan(3), class(section)],
@@ -862,14 +1099,47 @@ prolog:doc_file_index_header(File, Options) -->
 		   ]))).
 
 prolog:doc_object_title(Obj, Title) :-
-	Obj = section(_,_,_),
+	Obj = section(_,_,_,_),
 	man_index(Obj, Title, _, _, _), !.
 
-prolog:doc_canonical_object(section(Level, No, Path),
-			    section(Level, No, swi(Local))) :-
+prolog:doc_canonical_object(section(_Level, _No, ID, _Path),
+			    section(ID)).
+
+swi_local_path(Path, Local) :-
+	atom(Path),
 	is_absolute_file_name(Path),
-	absolute_file_name(swi(.), SWI,
+	absolute_file_name(swi(doc), SWI,
 			   [ file_type(directory),
 			     solutions(all)
 			   ]),
-	atom_concat(SWI, Local, Path), !.
+	directory_file_path(SWI, Local, Path), !.
+
+%%	prolog:doc_object_href(+Object, -HREF) is semidet.
+%
+%	Produce a HREF for section objects.
+
+prolog:doc_object_href(section(_Level, _No, ID, _Path), HREF) :-
+	atom_concat('sec:', Sec, ID),
+	http_link_to_id(pldoc_man, [section(Sec)], HREF).
+
+
+		 /*******************************
+		 *	     MESSAGES		*
+		 *******************************/
+
+:- multifile prolog:message//1.
+
+prolog:message(pldoc(no_section_id(File, Title))) -->
+	[ 'PlDoc: ~w: no id for section "~w"'-[File, Title] ].
+prolog:message(pldoc(duplicate_ids(L))) -->
+	[ 'PlDoc: duplicate manual section IDs:'-[], nl
+	],
+	duplicate_ids(L).
+
+duplicate_ids([]) --> [].
+duplicate_ids([H|T]) --> duplicate_id(H), duplicate_ids(T).
+
+duplicate_id(Id) -->
+	{ findall(File, man_index(section(_,_,Id,File),_,_,_,_), Files) },
+	[ '    ~w: ~p'-[Id, Files], nl ].
+
