@@ -1,14 +1,34 @@
-/*  $Id$
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker
-    E-mail:        jan@swi-prolog.org
+    E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 2006-2008, University of Amsterdam
+    Copyright (C): 2006-2013, University of Amsterdam
+			      VU University Amsterdam
 
-    This file is covered by the `The Artistic License', also in use by
-    Perl.  See http://www.perl.com/pub/a/language/misc/Artistic.html
+    This program is free software; you can redistribute it and/or
+    modify it under the terms of the GNU General Public License
+    as published by the Free Software Foundation; either version 2
+    of the License, or (at your option) any later version.
+
+    This program is distributed in the hope that it will be useful,
+    but WITHOUT ANY WARRANTY; without even the implied warranty of
+    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+    GNU General Public License for more details.
+
+    You should have received a copy of the GNU General Public
+    License along with this library; if not, write to the Free Software
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+    As a special exception, if you link this library with other files,
+    compiled with a Free Software compiler, to produce an executable, this
+    library does not by itself cause the resulting executable to be covered
+    by the GNU General Public License. This exception does not however
+    invalidate any other reasons why the executable file might be covered by
+    the GNU General Public License.
+
+    Alternatively, this program may be distributed under the Perl
+    Artistic License, version 2.0.
 */
 
 :- module(plunit,
@@ -29,9 +49,10 @@ Unit testing environment for SWI-Prolog and   SICStus Prolog. For usage,
 please visit http://www.swi-prolog.org/pldoc/package/plunit.html.
 
 @author		Jan Wielemaker
-@license	artistic
+@license	GPL+SWI-exception or Artistic 2.0
 */
 
+:- use_module(library(apply)).
 :- meta_predicate valid_options(+, 1).
 
 
@@ -79,7 +100,8 @@ user:term_expansion(In, Out) :-
 	prolog_load_context(module, plunit),
 	if_expansion(In, Out).
 
-swi     :- catch(current_prolog_flag(dialect, swi), _, fail).
+swi     :- catch(current_prolog_flag(dialect, swi), _, fail), !.
+swi     :- catch(current_prolog_flag(dialect, yap), _, fail).
 sicstus :- catch(current_prolog_flag(system_type, _), _, fail).
 
 
@@ -96,6 +118,12 @@ current_test_flag(Name, Value) :-
 
 set_test_flag(Name, Value) :-
 	create_prolog_flag(Name, Value, []).
+
+:- if(current_prolog_flag(dialect, yap)).
+
+'$set_predicate_attribute'(_, _, _).
+
+:- endif.
 :- endif.
 
 :- if(sicstus).
@@ -524,8 +552,10 @@ run_tests :-
 	cleanup,
 	setup_call_cleanup(
 	    setup_trap_assertions(Ref),
-	    forall(current_test_set(Set),
-		   run_unit(Set)),
+	    ( forall(current_test_set(Set),
+		     run_unit(Set)),
+	      check_for_test_errors
+	    ),
 	    ( cleanup_trap_assertions(Ref),
 	      report,
 	      cleanup_after_test
@@ -535,7 +565,9 @@ run_tests(Set) :-
 	cleanup,
 	setup_call_cleanup(
 	    setup_trap_assertions(Ref),
-	    run_unit(Set),
+	    ( run_unit(Set),
+	      check_for_test_errors
+	    ),
 	    ( cleanup_trap_assertions(Ref),
 	      report,
 	      cleanup_after_test
@@ -926,7 +958,7 @@ nondet_test(Expected, Unit, Name, Line, Options, Body, Result) :-
 		    Time is (T1 - T0)/1000.0,
 		    (   nondet_compare(Expected, Bindings, Unit, Name, Line)
 		    ->  Result = success(Unit, Name, Line, true, Time)
-		    ;   Result = failure(Unit, Name, Line, wrong_answer)
+		    ;   Result = failure(Unit, Name, Line, wrong_answer(Expected, Bindings))
 		    ),
 		    cleanup(Module, Options)
 		;   Result = failure(Unit, Name, Line, E),
@@ -1078,6 +1110,9 @@ success(Unit, Name, Line, Det, _Time, Options) :-
 	),
 	flush_output(user_error),
 	assert(fixme(Unit, Name, Line, Reason, Ok)).
+success(Unit, Name, Line, _, _, Options) :-
+	failed_assertion(Unit, Name, Line, _,_,_,_), !,
+	failure(Unit, Name, Line, assertion, Options).
 success(Unit, Name, Line, Det, Time, Options) :-
 	assert(passed(Unit, Name, Line, Det, Time)),
 	(   (   Det == true
@@ -1167,10 +1202,20 @@ running_tests(Running) :-
 		), Running).
 
 
-%%	report is semidet.
+%%	check_for_test_errors is semidet.
 %
-%	True if there are no errors.  If errors were encountered, report
-%	them to current output and fail.
+%	True if there are no errors, otherwise false.
+
+check_for_test_errors :-
+	number_of_clauses(failed/4, Failed),
+	number_of_clauses(failed_assertion/7, FailedAssertion),
+	number_of_clauses(sto/4, STO),
+	Failed+FailedAssertion+STO =:= 0.     % fail on errors
+
+
+%%	report is det.
+%
+%	Print a summary of the tests that ran.
 
 report :-
 	number_of_clauses(passed/5, Passed),
@@ -1185,9 +1230,10 @@ report :-
 	    info(plunit(all_passed(Passed)))
 	;   report_blocked,
 	    report_fixme,
-	    report_failed,
 	    report_failed_assertions,
-	    report_sto
+	    report_failed,
+	    report_sto,
+	    info(plunit(passed(Passed)))
 	).
 
 number_of_clauses(F/A,N) :-
@@ -1212,27 +1258,15 @@ report_blocked.
 
 report_failed :-
 	number_of_clauses(failed/4, N),
-	N > 0, !,
-	info(plunit(failed(N))),
-	fail.
-report_failed :-
-	info(plunit(failed(0))).
+	info(plunit(failed(N))).
 
 report_failed_assertions :-
 	number_of_clauses(failed_assertion/7, N),
-	N > 0, !,
-	info(plunit(failed_assertions(N))),
-	fail.
-report_failed_assertions :-
-	info(plunit(failed_assertions(0))).
+	info(plunit(failed_assertions(N))).
 
 report_sto :-
 	number_of_clauses(sto/4, N),
-	N > 0, !,
-	info(plunit(sto(N))),
-	fail.
-report_sto :-
-	info(plunit(sto(0))).
+	info(plunit(sto(N))).
 
 report_fixme :-
 	report_fixme(_,_,_).
@@ -1250,6 +1284,8 @@ fixme(How, Tuples, Count) :-
 	length(Tuples, Count).
 
 
+report_failure(_, _, _, assertion, _) :- !,
+	put_char(user_error, 'A').
 report_failure(Unit, Name, Line, Error, _Options) :-
 	print_message(error, plunit(failed(Unit, Name, Line, Error))).
 
@@ -1400,8 +1436,12 @@ message(plunit(blocked(Pos, Name, Reason))) -->
 					% fail/success
 message(plunit(no_tests)) --> !,
 	[ 'No tests to run' ].
+message(plunit(all_passed(1))) --> !,
+	[ 'test passed' ].
 message(plunit(all_passed(Count))) --> !,
 	[ 'All ~D tests passed'-[Count] ].
+message(plunit(passed(Count))) --> !,
+	[ '~D tests passed'-[Count] ].
 message(plunit(failed(0))) --> !,
 	[].
 message(plunit(failed(1))) --> !,
@@ -1587,6 +1627,21 @@ failure(wrong_answer(Cmp)) -->
 	},
 	[ 'wrong answer (compared using ~w)'-[Op], nl ],
 	expected_got_ops_(Ex, A, OPS, Goals).
+failure(wrong_answer(CmpExpected, Bindings)) -->
+	{ (   CmpExpected = all(Cmp)
+	  ->  Cmp =.. [Op,_,Expected],
+	      Got = Bindings,
+	      Type = all
+	  ;   CmpExpected = set(Cmp),
+	      Cmp =.. [Op,_,Expected0],
+	      sort(Expected0, Expected),
+	      sort(Bindings, Got),
+	      Type = set
+	  )
+	},
+	[ 'wrong "~w" answer:'-[Type] ],
+	[ nl, '    Expected: ~q'-[Expected] ],
+	[ nl, '       Found: ~q'-[Got] ].
 :- if(swi).
 failure(cmp_error(_Cmp, Error)) -->
 	{ message_to_string(Error, Message) },
