@@ -59,9 +59,9 @@ being flexible to ordering of tokens.
 	exclude_from_index/2.		% +Which, +Token
 
 
-setting(verbose(true)).			% print progress messages
+setting(verbose(false)).		% print progress messages
 setting(index_threads(1)).		% # threads for creating the index
-setting(index(default)).		% Use a thread for incremental updates
+setting(index(thread(1))).		% Use a thread for incremental updates
 
 %%	rdf_set_literal_index_option(+Options:list)
 %
@@ -232,23 +232,35 @@ expand_fuzzy(Var, _) :-
 	var(Var), !,
 	throw(error(instantiation_error, _)).
 expand_fuzzy(sounds(Like), Or) :- !,
-	metaphone_index(Map),
-	double_metaphone(Like, Key),
-	rdf_find_literal_map(Map, [Key], Tokens),
-	list_to_or(Tokens, sounds(Like), Or).
+	(   atom(Like)
+	->  metaphone_index(Map),
+	    double_metaphone(Like, Key),
+	    rdf_find_literal_map(Map, [Key], Tokens),
+	    list_to_or(Tokens, sounds(Like), Or)
+	;   expand_fuzzy(Like, Or)
+	).
 expand_fuzzy(stem(Like), Or) :- !,
-	porter_index(Map),
-	porter_stem(Like, Key),
-	rdf_find_literal_map(Map, [Key], Tokens),
-	list_to_or(Tokens, stem(Like), Or).
+	(   atom(Like)
+	->  porter_index(Map),
+	    porter_stem(Like, Key),
+	    rdf_find_literal_map(Map, [Key], Tokens),
+	    list_to_or(Tokens, stem(Like), Or)
+	;   expand_fuzzy(Like, Or)
+	).
 expand_fuzzy(prefix(Prefix), Or) :- !,
-	token_index(Map),
-	rdf_keys_in_literal_map(Map, prefix(Prefix), Tokens),
-	list_to_or(Tokens, prefix(Prefix), Or).
+	(   atom(Prefix)
+	->  token_index(Map),
+	    rdf_keys_in_literal_map(Map, prefix(Prefix), Tokens),
+	    list_to_or(Tokens, prefix(Prefix), Or)
+	;   expand_fuzzy(Prefix, Or)
+	).
 expand_fuzzy(case(String), Or) :- !,
-	token_index(Map),
-	rdf_keys_in_literal_map(Map, case(String), Tokens),
-	list_to_or(Tokens, case(String), Or).
+	(   atom(String)
+	->  token_index(Map),
+	    rdf_keys_in_literal_map(Map, case(String), Tokens),
+	    list_to_or(Tokens, case(String), Or)
+	;   expand_fuzzy(String, Or)
+	).
 expand_fuzzy(or(A0, B0), E) :- !,
 	expand_fuzzy(A0, A),
 	expand_fuzzy(B0, B),
@@ -378,11 +390,8 @@ token_index(Map) :-
 		    old_literal
 		  ],
 	(   setting(index(default))
-	->  (   current_prolog_flag(cpu_count, N), N > 1
-	    ->	create_update_literal_thread(1),
-		rdf_monitor(thread_monitor_literal, Monitor)
-	    ;	rdf_monitor(monitor_literal, Monitor)
-	    )
+	->  create_update_literal_thread(1),
+	    rdf_monitor(thread_monitor_literal, Monitor)
 	;   setting(index(thread(N)))
 	->  create_update_literal_thread(N),
 	    rdf_monitor(thread_monitor_literal, Monitor)
@@ -444,7 +453,7 @@ clean_token_index :-
 		 *	  THREADED UPDATE	*
 		 *******************************/
 
-%	create_update_literal_thread(+Threads)
+%%	create_update_literal_thread(+Threads)
 %
 %	Setup literal monitoring using threads.  While loading databases
 %	through rdf_attach_db/2 from  rdf_persistency.pl,   most  of the
@@ -459,25 +468,55 @@ create_update_literal_thread(Threads) :-
 			     [ alias(rdf_literal_monitor_queue),
 			       max_size(10000)
 			     ]),
-	forall(between(1, Threads, N),
-	       (   atom_concat(rdf_literal_monitor_, N, Alias),
-		   thread_create(monitor_literals, _,
-				 [ alias(Alias)
-				 ])
-	       )).
+	forall(between(1, Threads, _),
+	       create_index_worker(initial)).
 
-monitor_literals :-
+:- dynamic
+	index_worker_id/1.
+
+create_index_worker(Status) :-
+	(   retract(index_worker_id(Id0))
+	->  true
+	;   Id0 = 1
+	),
+	succ(Id0, Id1),
+	assertz(index_worker_id(Id1)),
+	atom_concat(rdf_literal_monitor_, Id0, Alias),
+	thread_create(monitor_literals(Status), _,
+		      [ alias(Alias)
+		      ]).
+
+monitor_literals(initial) :-
 	set_prolog_flag(agc_margin, 0),	% we don't create garbage
 	repeat,
 	    thread_get_message(rdf_literal_monitor_queue, Literal),
 	    register_literal(Literal),
 	fail.
+monitor_literals(extra) :-
+	set_prolog_flag(agc_margin, 0),
+	repeat,
+	    (	thread_get_message(rdf_literal_monitor_queue, Literal,
+				   [ timeout(1)
+				   ])
+	    ->	register_literal(Literal),
+		fail
+	    ;	!
+	    ),
+	thread_self(Me),
+	thread_detach(Me).
 
 thread_monitor_literal(new_literal(Literal)) :- !,
 	thread_send_message(rdf_literal_monitor_queue, Literal).
 thread_monitor_literal(Action) :- !,
 	monitor_literal(Action).
 
+check_index_workers(Alias) :-
+	message_queue_property(Queue, alias(Alias)),
+	message_queue_property(Queue, size(Size)),
+	Size > 5000, !,
+	with_mutex(create_index_worker, create_index_worker(extra)).
+check_index_workers(_).
+
 
 		 /*******************************
 		 *	 MONITORED UPDATE	*
@@ -512,7 +551,8 @@ add_tokens([H|T], Literal, Map) :-
 	->  true
 	;   forall(new_token(H), true),
 	    (	Keys mod 1000 =:= 0
-	    ->	progress(Map, 'Tokens')
+	    ->	progress(Map, 'Tokens'),
+		check_index_workers(rdf_literal_monitor_queue)
 	    ;	true
 	    )
 	),
@@ -622,11 +662,13 @@ fill_porter_index(PorterMap) :-
 stem([], _).
 stem([Token|T], Map) :-
 	(   atom(Token)
-	->  porter_stem(Token, Stem),
-	    rdf_insert_literal_map(Map, Stem, Token, Keys),
-	    (	integer(Keys),
-		Keys mod 1000 =:= 0
-	    ->  progress(Map, 'Porter')
+	->  (   porter_stem(Token, Stem)
+	    ->	rdf_insert_literal_map(Map, Stem, Token, Keys),
+		(   integer(Keys),
+		    Keys mod 1000 =:= 0
+		->  progress(Map, 'Porter')
+		;   true
+		)
 	    ;	true
 	    )
 	;   true
