@@ -1,11 +1,10 @@
-/*  $Id: sgml_write.pl,v 1.11 2006/02/16 11:32:04 jan Exp $
-
-    Part of SWI-Prolog
+/*  Part of SWI-Prolog
 
     Author:        Jan Wielemaker & Richard O'Keefe
-    E-mail:        wielemaker@science.uva.nl
+    E-mail:        J.Wielemaker@vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2004, University of Amsterdam
+    Copyright (C): 1985-2012, University of Amsterdam
+			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
     modify it under the terms of the GNU General Public License
@@ -19,7 +18,7 @@
 
     You should have received a copy of the GNU Lesser General Public
     License along with this library; if not, write to the Free Software
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+    Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
     As a special exception, if you link this library with other files,
     compiled with a Free Software compiler, to produce an executable, this
@@ -30,7 +29,7 @@
 */
 
 :- module(sgml_write,
-	  [ html_write/2,       	%          +Data, +Options
+	  [ html_write/2,		%          +Data, +Options
 	    html_write/3,		% +Stream, +Data, +Options
 	    sgml_write/2,		%          +Data, +Options
 	    sgml_write/3,		% +Stream, +Data, +Options
@@ -42,8 +41,26 @@
 :- use_module(library(debug)).
 :- use_module(library(assoc)).
 :- use_module(library(option)).
+:- use_module(library(error)).
+
+:- predicate_options(xml_write/2, 2, [pass_to(xml_write/3, 3)]).
+:- predicate_options(xml_write/3, 3,
+		     [ dtd(any),
+		       doctype(atom),
+		       public(atom),
+		       system(atom),
+		       header(boolean),
+		       nsmap(list),
+		       indent(nonneg),
+		       layout(boolean),
+		       net(boolean)
+		     ]).
+
+:- multifile
+	xmlns/2.			% NS, URI
+
+/** <module> XML/SGML writer module
 
-/* - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - -
 This library provides the inverse functionality   of  the sgml.pl parser
 library, writing XML, SGML and HTML documents from the parsed output. It
 is intended to allow rewriting in a  different dialect or encoding or to
@@ -57,15 +74,21 @@ very hazardous area.
 The Prolog-based low-level character and  escape   handling  is the real
 bottleneck in this library and will probably be   moved  to C in a later
 stage.
-- - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - - */
 
-%	xml_write( [+Stream,] +Data, +Options)
-%	sgml_write([+Stream,] +Data, +Options)
-%	html_write([+Stream,] +Data, +Options)
-%	
+@see	library(http/html_write) provides a high-level library for
+	emitting HTML and XHTML.
+*/
+
+%%	xml_write(+Data, +Options) is det.
+%%	sgml_write(+Data, +Options) is det.
+%%	html_write(+Data, +Options) is det.
+%%	xml_write(+Stream, +Data, +Options) is det.
+%%	sgml_write(+Stream, +Data, +Options) is det.
+%%	html_write(+Stream, +Data, +Options) is det.
+%
 %	Write a term as created by the SGML/XML parser to a stream in
 %	SGML or XML format.  Options:
-%	
+%
 %		* dtd(DTD)
 %		The DTD.  This is needed for SGML documents that contain
 %		elements with content model EMPTY.  Characters which may
@@ -88,36 +111,47 @@ stage.
 %		The public identifier to be written in the <!DOCTYPE> line.
 %
 %		* system(SysId)
-%		The system identifier to be written in the <!DOCTYPE> line. 
-%		
+%		The system identifier to be written in the <!DOCTYPE> line.
+%
 %		* header(Bool)
 %		If Bool is 'false', do not emit the <xml ...> header
 %		line.  (xml_write/3 only)
-%		
-%		* nsmap([Id=URI ...])
+%
+%		* nsmap(Map:list(Id=URI))
 %		When emitting embedded XML, assume these namespaces
 %		are already defined from the environment.  (xml_write/3
 %		only).
-%		
+%
 %		* indent(Indent)
 %		Indentation of the document (for embedding)
-%	
+%
 %		* layout(Bool)
 %		Emit/do not emit layout characters to make output
 %		readable.
-%	
+%
 %		* net(Bool)
 %		Use/do not use Null End Tags.
 %		For XML, this applies only to empty elements, so you get
+%
+%		==
 %		    <foo/>	(default, net(true))
 %		    <foo></foo>	(net(false))
+%		==
+%
 %		For SGML, this applies to empty elements, so you get
+%
+%		==
 %		    <foo>	(if foo is declared to be EMPTY in the DTD)
 %		    <foo></foo>	(default, net(false))
 %		    <foo//	(net(true))
+%		==
+%
 %		and also to elements with character content not containing /
+%
+%		==
 %		    <b>xxx</b>	(default, net(false))
 %		    <b/xxx/	(net(true)).
+%		==
 %
 %	Note that if the stream is UTF-8, the system will write special
 %	characters as UTF-8 sequences, while if it is ISO Latin-1 it
@@ -193,33 +227,24 @@ update_state(dtd(DTD), State) :- !,
 update_state(nsmap(Map), State) :- !,
 	set_state(State, nsmap, Map).
 update_state(indent(Indent), State) :- !,
-	must_be(Indent, integer),
+	must_be(integer, Indent),
 	set_state(State, indent, Indent).
 update_state(layout(Bool), State) :- !,
-	must_be(Bool, bool),
+	must_be(boolean, Bool),
 	set_state(State, layout, Bool).
 update_state(doctype(_), _) :- !.
 update_state(public(_),  _) :- !.
 update_state(system(_),  _) :- !.
 update_state(net(Bool), State) :- !,
-	must_be(Bool, bool),
+	must_be(boolean, Bool),
 	set_state(State, net, Bool).
 update_state(header(Bool), _) :- !,
-	must_be(Bool, bool).
+	must_be(boolean, Bool).
 update_state(Option, _) :-
-	throw(error(domain_error(xml_write_option, Option), _)).
-
-must_be(Arg, Type) :-
-	call(Type, Arg), !.
-must_be(Arg, Type) :-
-	throw(error(type_error(Type, Arg), _)).
-
-bool(true).
-bool(false).
-
+	domain_error(xml_write_option, Option).
 
 %	emit_xml_encoding(+Stream, +Options)
-%	
+%
 %	Emit the XML fileheader with   encoding information. Setting the
 %	right encoding on the output stream  must be done before calling
 %	xml_write/3.
@@ -228,17 +253,19 @@ emit_xml_encoding(Out, Options) :-
 	option(header(Hdr), Options, true),
 	Hdr == true, !,
 	stream_property(Out, encoding(Encoding)),
-	(   Encoding == utf8
+	(   (   Encoding == utf8
+	    ;	Encoding == wchar_t
+	    )
 	->  format(Out, '<?xml version="1.0" encoding="UTF-8"?>~n~n', [])
 	;   Encoding == iso_latin_1
 	->  format(Out, '<?xml version="1.0" encoding="ISO-8859-1"?>~n~n', [])
-	;   throw(error(domain_error(xml_encoding, Encoding), _))
+	;   domain_error(xml_encoding, Encoding)
 	).
 emit_xml_encoding(_, _).
 
 
-%	emit_doctype(+Options, +Data, +Stream)
-%	
+%%	emit_doctype(+Options, +Data, +Stream)
+%
 %	Emit the document-type declaration.
 %	There is a problem with the first clause if we are emitting SGML:
 %	the SGML DTDs for HTML up to HTML 4 *do not allow* any 'version'
@@ -280,18 +307,24 @@ write_doctype(Out, DocType, PubId, SysId) :-
 	format(Out, '<!DOCTYPE ~w PUBLIC "~w" "~w">~n~n', [DocType,PubId,SysId]).
 
 
-%	emit(+Element, +Out, +State, +Options)
-%	
+%%	emit(+Element, +Out, +State, +Options)
+%
 %	Emit a single element
 
+emit(Var, _, _) :-
+	var(Var), !,
+	instantiation_error(Var).
 emit([], _, _) :- !.
 emit([H|T], Out, State) :- !,
 	emit(H, Out, State),
 	emit(T, Out, State).
+emit(CDATA, Out, State) :-
+	atomic(CDATA), !,
+	sgml_write_content(Out, CDATA, State).
 emit(Element, Out, State) :-
 	\+ \+ emit_element(Element, Out, State).
 
-emit_element(pi(PI), Out, State) :-
+emit_element(pi(PI), Out, State) :- !,
 	get_state(State, entity_map, EntityMap),
 	write(Out, <?),
 	write_quoted(Out, PI, "", EntityMap),
@@ -299,7 +332,9 @@ emit_element(pi(PI), Out, State) :-
 	    write(Out, ?>)
 	;   write(Out, >)
 	).
-emit_element(element(Name, Attributes, Content), Out, State) :-
+emit_element(element(Name, Attributes, Content), Out, State) :- !,
+	must_be(list, Attributes),
+	must_be(list, Content),
 	att_length(Attributes, State, Alen),
 	(   Alen > 60,
 	    get_state(State, layout, true)
@@ -321,6 +356,8 @@ emit_element(element(Name, Attributes, Content), Out, State) :-
 	;   attributes(Attributes, Sep, Out, State)
 	),
 	content(Content, Out, Name, State).
+emit_element(E, _, _) :-
+	type_error(xml_dom, E).
 
 attributes([], _, _, _).
 attributes([H|T], Sep, Out, State) :-
@@ -347,7 +384,7 @@ att_length([A0|T], State, Len0, Len) :-
 
 alen(URI:Name=Value, State, Len) :- !,
 	atom_length(Value, AL),
-	atom_length(Name, NL),
+	vlen(Name, NL),
 	get_state(State, nsmap, Nsmap),
 	(   memberchk(NS=URI, Nsmap)
 	->  atom_length(NS, NsL)
@@ -356,9 +393,25 @@ alen(URI:Name=Value, State, Len) :- !,
 	Len is AL+NL+NsL+3.
 alen(Name=Value, _, Len) :-
 	atom_length(Name, NL),
-	atom_length(Value, AL),
+	vlen(Value, AL),
 	Len is AL+NL+3.
 
+vlen(Value, Len) :-
+	is_list(Value), !,
+	vlen_list(Value, 0, Len).
+vlen(Value, Len) :-
+	atom_length(Value, Len).
+
+vlen_list([], L, L).
+vlen_list([H|T], L0, L) :-
+	atom_length(H, HL),
+	(   L0 == 0
+	->  L1 is L0 + HL
+	;   L1 is L0 + HL + 1
+	),
+	vlen_list(T, L1, L).
+
+
 emit_name(Name, Out, _) :-
 	atom(Name), !,
 	write(Out, Name).
@@ -369,11 +422,11 @@ emit_name(URI:Name, Out, State) :-
 	->  write(Out, Name)
 	;   format(Out, '~w:~w', [NS, Name])
 	).
-emit_name(Term, Out, _) :-
+emit_name(Term, Out, _) :-		% error?
 	write(Out, Term).
 
-%	update_nsmap(+Attributes, !State)
-%	
+%%	update_nsmap(+Attributes, !State)
+%
 %	Modify the nsmap of State to reflect modifications due to xmlns
 %	arguments.
 
@@ -398,8 +451,8 @@ set_nsmap(NS, URI, Map0, Map) :-
 set_nsmap(NS, URI, Map, [NS=URI|Map]).
 
 
-%	content(+Content, +Out, +Element, +State, +Options)
-%	
+%%	content(+Content, +Out, +Element, +State, +Options)
+%
 %	Emit the content part of a structure  as well as the termination
 %	for the content. For empty content   we have three versions: XML
 %	style '/>', SGML declared EMPTY element (nothing) or normal SGML
@@ -421,19 +474,19 @@ content([], Out, Element, State) :- !,	% empty element
 	;   emit_close(Element, Out, State)
 	)
     ).
-content([Atom], Out, Element, State) :-
-	atom(Atom), !,
+content([CDATA], Out, Element, State) :-
+	atomic(CDATA), !,
 	(   get_state(State, dialect, sgml),
 	    get_state(State, net, true),
-	    \+ sub_atom(Atom, _, _, _, /),
-	    atom_length(Atom, Len),
+	    \+ sub_atom(CDATA, _, _, _, /),
+	    write_length(CDATA, Len, []),
 	    Len < 20
 	->  write(Out, /),
-	    sgml_write_content(Out, Atom, State),
+	    sgml_write_content(Out, CDATA, State),
 	    write(Out, /)
 	;/* XML or not NET */
 	    write(Out, >),
-	    sgml_write_content(Out, Atom, State),
+	    sgml_write_content(Out, CDATA, State),
 	    emit_close(Element, Out, State)
 	).
 content(Content, Out, Element, State) :-
@@ -508,8 +561,8 @@ write_element_content([H|T], Out, State) :-
 		 *	     NAMESPACES		*
 		 *******************************/
 
-%	add_missing_namespaces(+DOM0, +NsMap, -DOM)
-%	
+%%	add_missing_namespaces(+DOM0, +NsMap, -DOM)
+%
 %	Add xmlns:NS=URI definitions to the toplevel element(s) to
 %	deal with missing namespaces.
 
@@ -532,15 +585,49 @@ add_missing_ns([H|T], Atts0, Atts) :-
 	generate_ns(H, NS),
 	add_missing_ns(T, [xmlns:NS=H|Atts0], Atts).
 
+%%	generate_ns(+URI, -NS) is det.
+%
+%	Generate a namespace (NS) identifier for URI.
+
+generate_ns(URI, NS) :-
+	xmlns(NS, URI), !.
 generate_ns(URI, NS) :-
 	default_ns(URI, NS), !.
 generate_ns(_, NS) :-
 	gensym(xns, NS).
 
-default_ns('http://www.w3.org/2001/XMLSchema-instance', xsi).
+%%	xmlns(?NS, ?URI) is nondet.
+%
+%	Hook to define human readable  abbreviations for XML namespaces.
+%	xml_write/3 tries these locations:
+%
+%	  1. This hook
+%	  2. Defaults (see below)
+%	  3. rdf_db:ns/2 for RDF-DB integration
+%
+%	Default XML namespaces are:
+%
+%	  | xsi    | http://www.w3.org/2001/XMLSchema-instance |
+%	  | xs     | http://www.w3.org/2001/XMLSchema          |
+%	  | xhtml  | http://www.w3.org/1999/xhtml	       |
+%	  | soap11 | http://schemas.xmlsoap.org/soap/envelope/ |
+%	  | soap12 | http://www.w3.org/2003/05/soap-envelope   |
+%
+%	@see xml_write/2, rdf_register_ns/2.
 
-%	missing_namespaces(+DOM, +NSMap, -Missing)
-%	
+:- multifile
+	rdf_db:ns/2.
+
+default_ns('http://www.w3.org/2001/XMLSchema-instance', xsi).
+default_ns('http://www.w3.org/2001/XMLSchema',		xs).
+default_ns('http://www.w3.org/1999/xhtml',		xhtml).
+default_ns('http://schemas.xmlsoap.org/soap/envelope/', soap11).
+default_ns('http://www.w3.org/2003/05/soap-envelope',   soap12).
+default_ns(URI, NS) :-
+	rdf_db:ns(NS, URI).
+
+%%	missing_namespaces(+DOM, +NSMap, -Missing)
+%
 %	Return a list of URIs appearing in DOM that are not covered
 %	by xmlns definitions.
 
@@ -582,13 +669,19 @@ sgml_write_attribute(Out, Values, State) :-
 	is_list(Values), !,
 	get_state(State, entity_map, EntityMap),
 	put_char(Out, '"'),
-	write_quoted_list(Values, Out, """&", EntityMap),
+	write_quoted_list(Values, Out, """<&>", EntityMap),
 	put_char(Out, '"').
 sgml_write_attribute(Out, Value, State) :-
+	is_text(Value), !,
 	get_state(State, entity_map, EntityMap),
 	put_char(Out, '"'),
-	write_quoted(Out, Value, """&", EntityMap),
+	write_quoted(Out, Value, """<&>", EntityMap),
 	put_char(Out, '"').
+sgml_write_attribute(Out, Value, _State) :-
+	number(Value), !,
+	write(Out, Value).
+sgml_write_attribute(_, Value, _) :-
+	type_error(sgml_attribute_value, Value).
 
 write_quoted_list([], _, _, _).
 write_quoted_list([H|T], Out, Escape, EntityMap) :-
@@ -601,13 +694,25 @@ write_quoted_list([H|T], Out, Escape, EntityMap) :-
 
 
 sgml_write_content(Out, Value, State) :-
+	is_text(Value), !,
 	get_state(State, entity_map, EntityMap),
 	write_quoted(Out, Value, "<&>", EntityMap).
+sgml_write_content(Out, Value, _) :-
+	write(Out, Value).
 
+is_text(Value) :- atom(Value), !.
+is_text(Value) :- string(Value), !.
 
 write_quoted(Out, Atom, Escape, EntityMap) :-
+	atom(Atom), !,
 	atom_codes(Atom, Codes),
 	writeq(Codes, Out, Escape, EntityMap).
+write_quoted(Out, String, Escape, EntityMap) :-
+	string(String), !,
+	string_to_list(String, Codes),
+	writeq(Codes, Out, Escape, EntityMap).
+write_quoted(_, String, _, _) :-
+	type_error(atom_or_string, String).
 
 
 writeq([], _, _, _).
@@ -615,7 +720,8 @@ writeq([H|T], Out, Escape, EntityMap) :-
 	(   memberchk(H, Escape)
 	->  write_entity(H, Out, EntityMap)
 	;   H >= 256
-	->  (   stream_property(Out, encoding(utf8))
+	->  (   stream_property(Out, encoding(Enc)),
+		unicode_encoding(Enc)
 	    ->	put_code(Out, H)
 	    ;	write_entity(H, Out, EntityMap)
 	    )
@@ -623,6 +729,10 @@ writeq([H|T], Out, Escape, EntityMap) :-
 	),
 	writeq(T, Out, Escape, EntityMap).
 
+unicode_encoding(utf8).
+unicode_encoding(wchar_t).
+unicode_encoding(unicode_le).
+unicode_encoding(unicode_be).
 
 write_entity(Code, Out, EntityMap) :-
 	(   get_assoc(Code, EntityMap, EntityName)
@@ -654,7 +764,7 @@ emit_indent(Indent, Out) :-
 	format(Out, '~N', []),
 	write_n(Tabs, '\t', Out),
 	write_n(Spaces, ' ', Out).
-	
+
 write_n(N, Char, Out) :-
 	(   N > 0
 	->  put_char(Out, Char),
@@ -662,7 +772,7 @@ write_n(N, Char, Out) :-
 	    write_n(N2, Char, Out)
 	;   true
 	).
-	
+
 inc_indent(State) :-
 	inc_indent(State, 2).
 
@@ -677,8 +787,8 @@ inc_indent(State, Inc) :-
 		 *	   DTD HANDLING		*
 		 *******************************/
 
-%	empty_element(+State, +Element)
-%	
+%%	empty_element(+State, +Element)
+%
 %	True if Element is declared  with   EMPTY  content in the (SGML)
 %	DTD.
 
@@ -686,9 +796,9 @@ empty_element(State, Element) :-
 	get_state(State, dtd, DTD),
 	DTD \== (-),
 	dtd_property(DTD, element(Element, _, empty)).
-	
-%	dtd_character_entities(+DTD, -Map)
-%	
+
+%%	dtd_character_entities(+DTD, -Map)
+%
 %	Return an assoc mapping character entities   to their name. Note
 %	that the entity representation is a bit dubious. Entities should
 %	allow for a wide-character version and avoid the &#..; trick.
@@ -738,8 +848,12 @@ new_state(Dialect,
 	[],		% NS=Full map
 	Net		% Null End-Tags?
     )) :-
-	( Dialect == sgml -> Net = false ; Net = true ),
-	empty_assoc(EntityMap).
+	(   Dialect == sgml
+	->  Net = false,
+	    empty_assoc(EntityMap)
+	;   Net = true,
+	    xml_entities(EntityMap)
+	).
 
 get_state(State, Field, Value) :-
 	state(Field, Arg),
@@ -749,6 +863,14 @@ set_state(State, Field, Value) :-
 	state(Field, Arg),
 	setarg(Arg, State, Value).
 
+xml_entities(Map) :-
+	list_to_assoc([ 60 - lt,
+			61 - amp,
+			62 - gt,
+			39 - apos,
+			34 - quot
+		      ], Map).
+
 
 		 /*******************************
 		 *	      MESSAGES		*
