@@ -1,11 +1,9 @@
-/*  $Id$
-
-    Part of XPCE --- The SWI-Prolog GUI toolkit
+/*  Part of XPCE --- The SWI-Prolog GUI toolkit
 
     Author:        Jan Wielemaker and Anjo Anjewierden
     E-mail:        J.Wielemaker@cs.vu.nl
     WWW:           http://www.swi-prolog.org
-    Copyright (C): 1985-2011, University of Amsterdam
+    Copyright (C): 1985-2012, University of Amsterdam
 			      VU University Amsterdam
 
     This program is free software; you can redistribute it and/or
@@ -156,6 +154,10 @@ open_query_window(IDE) :->
 	),
 	send(QF, expose).
 
+open_interactor(_) :->
+	"Create a new interactor window"::
+	interactor.
+
 thread_monitor(IDE) :->
 	"Open a monitor for running threads"::
 	(   current_prolog_flag(threads, true)
@@ -166,7 +168,7 @@ thread_monitor(IDE) :->
 	    ),
 	    send(Monitor, open)
 	;   send(@display, report, error,
-		 'This version of SWI-Prolog is not built\n\
+		 'This version of SWI-Prolog is not built\n\c
 		  with thread-support')
 	).
 
