@@ -55,7 +55,7 @@ extern "C" {
 /* PLVERSION: 10000 * <Major> + 100 * <Minor> + <Patch> */
 
 #ifndef PLVERSION
-#define PLVERSION 60205
+#define PLVERSION 60319
 #endif
 
 		 /*******************************
@@ -395,7 +395,7 @@ PL_EXPORT(int)		PL_get_bool(term_t t, int *value) WUNUSED;
 PL_EXPORT(int)		PL_get_atom_chars(term_t t, char **a) WUNUSED;
 #define PL_get_string_chars(t, s, l) PL_get_string(t,s,l)
 					/* PL_get_string() is deprecated */
-PL_EXPORT(int)		PL_get_string(term_t t, char **s, size_t *len); WUNUSED
+PL_EXPORT(int)		PL_get_string(term_t t, char **s, size_t *len) WUNUSED;
 PL_EXPORT(int)		PL_get_chars(term_t t, char **s, unsigned int flags) WUNUSED;
 PL_EXPORT(int)		PL_get_list_chars(term_t l, char **s,
 					  unsigned int flags) WUNUSED;
@@ -457,7 +457,7 @@ PL_EXPORT(int)		PL_put_pointer(term_t t, void *ptr) WUNUSED;
 PL_EXPORT(int)		PL_put_float(term_t t, double f) WUNUSED;
 PL_EXPORT(int)		PL_put_functor(term_t t, functor_t functor) WUNUSED;
 PL_EXPORT(int)		PL_put_list(term_t l) WUNUSED;
-PL_EXPORT(void)		PL_put_nil(term_t l);
+PL_EXPORT(int)		PL_put_nil(term_t l);
 PL_EXPORT(int)		PL_put_term(term_t t1, term_t t2);
 
 			/* construct a functor or list-cell */
@@ -785,9 +785,11 @@ PL_EXPORT(IOSTREAM *)	PL_open_resource(module_t m,
 
 PL_EXPORT(IOSTREAM *)*_PL_streams(void);	/* base of streams */
 #ifndef PL_KERNEL
-#define Suser_input  (_PL_streams()[0])
-#define Suser_output (_PL_streams()[1])
-#define Suser_error  (_PL_streams()[2])
+#define Suser_input     (_PL_streams()[0])
+#define Suser_output    (_PL_streams()[1])
+#define Suser_error     (_PL_streams()[2])
+#define Scurrent_input  (_PL_streams()[3])
+#define Scurrent_output (_PL_streams()[4])
 #endif
 
 #define PL_WRT_QUOTED		0x01	/* quote atoms */
@@ -810,6 +812,7 @@ PL_EXPORT(IOSTREAM *)*_PL_streams(void);	/* base of streams */
 #define PL_WRT_NO_CYCLES	0x800	/* Never emit @(Template,Subst) */
 #define PL_WRT_LIST	       0x1000	/* Write [...], even with ignoreops */
 #define PL_WRT_NEWLINE	       0x2000	/* Add a newline */
+#define PL_WRT_VARNAMES	       0x4000	/* Internal: variable_names(List)  */
 
 PL_EXPORT(int) PL_write_term(IOSTREAM *s,
 			     term_t term,
@@ -944,6 +947,9 @@ PL_EXPORT(int)	PL_get_signum_ex(term_t sig, int *n);
 PL_EXPORT(int)	PL_action(int, ...);	/* perform some action */
 PL_EXPORT(void) PL_on_halt(void (*)(int, void *), void *);
 PL_EXPORT(void) PL_backtrace(int depth, int flags);
+PL_EXPORT(int)	PL_check_data(term_t data);
+PL_EXPORT(int)	PL_current_prolog_flag(atom_t name, int type, void *ptr);
+
 
 		/********************************
 		*         QUERY PROLOG          *
