@@ -470,9 +470,7 @@ lookupBlob(const char *s, size_t length, PL_blob_t *type, int *new)
 #ifdef O_ATOMGC
   if ( GD->atoms.margin != 0 &&
        GD->atoms.unregistered >= GD->atoms.non_garbage + GD->atoms.margin )
-  { intptr_t x = GD->atoms.unregistered - (GD->atoms.non_garbage + GD->atoms.margin);
-
-    if ( x % 128 == 0 )			/* see (*) above */
+  { if ( GD->statistics.atoms % 128 == 0 ) /* see (*) above */
       PL_raise(SIG_ATOM_GC);
   }
 #endif
@@ -791,6 +789,7 @@ pl_garbage_collect_atoms(void)
 
   PL_LOCK(L_THREAD);
   PL_LOCK(L_AGC);
+  PL_LOCK(L_STOPTHEWORLD);
   LOCK();
   GD->atoms.gc_active = TRUE;
   blockSignals(&set);
@@ -810,6 +809,7 @@ pl_garbage_collect_atoms(void)
   GD->atoms.gc++;
   unblockSignals(&set);
   UNLOCK();
+  PL_UNLOCK(L_STOPTHEWORLD);
   PL_UNLOCK(L_AGC);
   PL_UNLOCK(L_THREAD);
   gc_status.blocked--;
