@@ -77,7 +77,6 @@ static const PL_extension foreigns[] = {
   FRG("getenv",			2, pl_getenv,			0),
   FRG("setenv",			2, pl_setenv,			0),
   FRG("unsetenv",		1, pl_unsetenv,			0),
-  FRG("$apropos_match",		2, pl_apropos_match,		0),
   FRG("sub_atom",		5, pl_sub_atom,		 NDET|ISO),
   FRG("sleep",			1, pl_sleep,			0),
   FRG("break",			0, pl_break,			0),
@@ -384,6 +383,7 @@ DECL_PLIST(termhash);
 DECL_PLIST(dde);
 DECL_PLIST(term);
 DECL_PLIST(debug);
+DECL_PLIST(locale);
 
 void
 initBuildIns(void)
@@ -435,6 +435,9 @@ initBuildIns(void)
 #ifdef __WINDOWS__
   REG_PLIST(win);
   REG_PLIST(dde);
+#endif
+#ifdef O_LOCALE
+  REG_PLIST(locale);
 #endif
   REG_PLIST(debug);
 
@@ -490,6 +493,7 @@ initBuildIns(void)
   PL_meta_predicate(PL_predicate("thread_signal",    2, "system"), "+0");
 #endif
   PL_meta_predicate(PL_predicate("prolog_frame_attribute", 3, "system"), "++:");
+  PL_meta_predicate(PL_predicate("compile_predicates", 1, "system"), ":");
 
   for( ecell = ext_head; ecell; ecell = ecell->next )
     bindExtensions(ecell->module, ecell->extensions);
